/*
 * Bull SAS / OW2 Orchestra
 * Copyright (C) 2011 Bull S.A.S, and individual contributors as indicated
 * by the @authors tag.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

/**
 * @author Jerome Camilleri
 */
package org.ow2.orchestra.designer.view.dataTypeEditor {
import mx.binding.utils.BindingUtils;
import mx.collections.ArrayCollection;
import mx.containers.Box;
import mx.controls.Label;
import mx.events.CollectionEvent;
import mx.states.AddChild;
import mx.states.State;

import org.ow2.orchestra.designer.bpmn.model.AbstractElement;
import org.ow2.orchestra.designer.bpmn.model.data.type.SequenceType;
import org.ow2.orchestra.designer.bpmn.model.data.type.SimpleType;
import org.ow2.orchestra.designer.bpmn.model.data.type.WSDLMessageModel;
import org.ow2.orchestra.designer.bpmn.model.data.type.WSDLMessagePartModel;
import org.ow2.orchestra.designer.bpmn.model.process.EventModel;
import org.ow2.orchestra.designer.bpmn.model.process.EventType;
import org.ow2.orchestra.designer.model.DesignerModelLocator;
import org.ow2.orchestra.designer.model.ProcessEditorModel;

public class DataTypeEditorView extends Box {
  /**
   * The source for "properties" image data binding.
   */
  [Bindable]
  [Embed(source="/../assets/icon/container/properties.gif")]
  public var dataTypeIcon:Class;

  private var _dataTypePanel:DataTypeUI = new DataTypeUI();
  private var _processEditorModel:ProcessEditorModel;
  private var selectedDataType:WSDLMessageModel;

  private static const DISABLED_STATE_NAME:String = "disabled";
  private static const ENABLED_STATE_NAME:String = "enabled";

  public function DataTypeEditorView() {
    super();

    this.percentHeight = 100;
    this.percentWidth = 100;
    this.label = "Data Types";
    this.icon = dataTypeIcon;

    _dataTypePanel.percentHeight = 100;
    _dataTypePanel.percentWidth = 100;

    var enabledState:State = new State();
    enabledState.name = ENABLED_STATE_NAME;
    enabledState.overrides.push(new AddChild(this, _dataTypePanel));
    this.states.push(enabledState);

    var disabledState:State = new State();
    disabledState.name = DISABLED_STATE_NAME;
    var disabledLabel:Label = new Label();
    disabledLabel.text = "Data editor not available for the selected element.";
    disabledLabel.percentHeight = 100;
    disabledLabel.percentWidth = 100;
    disabledState.overrides.push(new AddChild(this, disabledLabel));
    this.states.push(disabledState);

    this.currentState = DISABLED_STATE_NAME;

    // Active the tab matching to the selected process
    BindingUtils.bindSetter(setSelectedProcessEditor, DesignerModelLocator.getInstance(), "selectedProcessEditor");
  }

  /**
   * Active the tab matching to the selected process
   */
  private function setSelectedProcessEditor(selectedProcessEditor:ProcessEditorModel):void {
    if (_processEditorModel != null) {
      _processEditorModel.selectedElements.removeEventListener(CollectionEvent.COLLECTION_CHANGE, selectedElementChanged);
      this.releaseSelectedDataType();
    }
    _processEditorModel = selectedProcessEditor;
    if (selectedProcessEditor != null) {
      selectedProcessEditor.selectedElements.addEventListener(CollectionEvent.COLLECTION_CHANGE, selectedElementChanged);
    }
    this.selectedElementChanged();
  }

  /**
   * When the collection is modified then we update the data grid.
   * Note: when a element of the collection is updated (one of its values changed) the a collection event
   * {@link mx.events.CollectionEventKind@UPDATE} is dispatched. So we do not have to listen on properties changed events for
   * selected
   * elements.
   * @param event A collection event
   */
  private function selectedElementChanged(event:CollectionEvent = null):void {
    if (_processEditorModel != null) {
      if (_processEditorModel.selectedElements.length == 1) {
        var selectedElement:AbstractElement = _processEditorModel.selectedElements.getItemAt(0) as AbstractElement;
        if (selectedElement is EventModel) {
          var selectedEvent:EventModel = selectedElement as EventModel;
          this.releaseSelectedDataType();
          if (selectedEvent.eventType == EventType.RECEIVE_MESSAGE
              && (selectedEvent.operation != null && selectedEvent.operation.inMessageType != null)) {
            selectedDataType = selectedEvent.operation.inMessageType as WSDLMessageModel;
          }
          else if (selectedEvent.eventType == EventType.SEND_MESSAGE
              && (selectedEvent.operation != null && selectedEvent.operation.outMessageType != null)) {
            selectedDataType = selectedEvent.operation.outMessageType as WSDLMessageModel;
          }

          if (selectedDataType != null && selectedDataType.parts.length >= 1) {
            //We must first set the current state to enable in order to create UI
            //Otherwise _dataTypePanel.typeEditor doesn't exist
            this.currentState = ENABLED_STATE_NAME;
            selectedDataType.parts.addEventListener(CollectionEvent.COLLECTION_CHANGE, updateDataProvider);
            this.updateDataProvider();

          }  else {
            this.currentState = DISABLED_STATE_NAME;
          }
        } else {
          this.releaseSelectedDataType();
          this.currentState = DISABLED_STATE_NAME;
        }
      } else {
        this.releaseSelectedDataType();
        this.currentState = DISABLED_STATE_NAME;
      }
    } else {
      this.releaseSelectedDataType();
      this.currentState = DISABLED_STATE_NAME;
    }
  }

  private function updateDataProvider(event:CollectionEvent = null):void {
    if (selectedDataType != null && selectedDataType.parts.length > 0) {
      var partModel:WSDLMessagePartModel = selectedDataType.parts.getItemAt(0) as WSDLMessagePartModel;
      if (partModel != null) {
        if (partModel.xsdElement != null && partModel.xsdElement.type is SimpleType) {
          _dataTypePanel.typeEditor.dataProvider = new ArrayCollection();
          (_dataTypePanel.typeEditor.dataProvider as ArrayCollection).addItem(partModel.xsdElement);
        } else if (partModel.xsdType != null && partModel.xsdType is SequenceType) {
          _dataTypePanel.typeEditor.dataProvider = SequenceType(partModel.xsdType).elements;
        } else if (partModel.xsdElement != null && partModel.xsdElement.type is SequenceType) {
          _dataTypePanel.typeEditor.dataProvider = SequenceType(partModel.xsdElement.type).elements;
        }
      }
    } else {
      _dataTypePanel.typeEditor.dataProvider = new ArrayCollection();
    }
  }


  private function releaseSelectedDataType():void {
    if (selectedDataType != null) {
      selectedDataType.parts.removeEventListener(CollectionEvent.COLLECTION_CHANGE, updateDataProvider);
      selectedDataType = null;
      if (_dataTypePanel.typeEditor != null) {
        _dataTypePanel.typeEditor.dataProvider = new ArrayCollection();
      }
    }
  }
}
}
