/**
 * Bull SAS / OW2 Orchestra
 * Copyright (C) 2010 Bull S.A.S, and individual contributors as indicated
 * by the @authors tag.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */
package org.ow2.orchestra.designer.view.diagramEditor {
import com.adobe.cairngorm.CairngormError;
import com.adobe.cairngorm.CairngormMessageCodes;

import flash.display.DisplayObject;
import flash.display.InteractiveObject;
import flash.events.Event;
import flash.events.KeyboardEvent;
import flash.events.MouseEvent;
import flash.geom.Point;
import flash.ui.Keyboard;

import mx.containers.Canvas;
import mx.controls.Image;
import mx.core.Application;
import mx.core.EventPriority;
import mx.events.CloseEvent;
import mx.events.CollectionEvent;
import mx.events.CollectionEventKind;
import mx.events.DragEvent;
import mx.managers.IFocusManagerComponent;
import mx.managers.PopUpManager;

import org.ow2.orchestra.designer.bpmn.model.AbstractElement;
import org.ow2.orchestra.designer.bpmn.model.AbstractElementWithPosition;
import org.ow2.orchestra.designer.bpmn.views.AbstractElementView;
import org.ow2.orchestra.designer.bpmn.views.AbstractDiagramView;
import org.ow2.orchestra.designer.bpmn.views.utils.Arrow;
import org.ow2.orchestra.designer.events.DiagramEvent;
import org.ow2.orchestra.designer.model.DiagramEditorModel;
import org.ow2.orchestra.designer.view.ConfirmPopup;
import org.ow2.orchestra.designer.view.ElementContextPanel;

/**
 *
 * @author hallouin-s
 *
 * This class implements @see(IFocusManagerComponent) to gets the focus
 * on click. This will be used to intercept some events like Keybord events.
 */
public class AbstractDiagramEditorView extends Canvas implements IFocusManagerComponent {

  private var diagramSelectionViewer:DiagramEditorSelectionViewer;

  private var deleteConfirmWindow:ConfirmPopup;

  internal var elementContextPanel:ElementContextPanel;

  /**
   * selection rectangle object.
   * In the "selection" state, defines the selected area.
   * In the "moving" state, defines the previous mouse position (used to update elements positions).
   */
  internal var selectionRect:Canvas;

  /**
   * Variable used for creatingFlow and creatingElement states.
   */
  public var arrow:Arrow;
  public var imageOfElementToAdd:Image = new Image();
  public var modelOfElementToAdd:AbstractElementWithPosition;

  /** diagram view should accept dropping of the current object */
  protected var acceptDrop:Boolean;
  /** pre-validation of drag is correct */
  protected var dragEnterValid:Boolean;


  protected var _diagramView:AbstractDiagramView;

  [Bindable]
  protected var _diagramEditorModel:DiagramEditorModel;


  protected var _viewXOffset:int = 0;
  protected var _viewYOffset:int = 0;

  /**
   *
   * @param diagramEditorModel
   */
  public function AbstractDiagramEditorView(diagramEditorModel:DiagramEditorModel) {

    _diagramEditorModel = diagramEditorModel;

    _diagramEditorModel.selectedElements.addEventListener(CollectionEvent.COLLECTION_CHANGE,
        modelSelectedElementsChanged, false, EventPriority.DEFAULT, true);

    elementContextPanel = new ElementContextPanel(this);

    this.setStyle("borderStyle", "solid");
    this.setStyle("backgroundColor", "white");

    // add drag enter and drop listeners for objects dragged from the palette
    this.addEventListener(DragEvent.DRAG_ENTER, diagramViewDragEnter, false, EventPriority.DEFAULT, true);
    this.addEventListener(DragEvent.DRAG_DROP, diagramViewDrop, false, EventPriority.DEFAULT, true);
    this.addEventListener(DragEvent.DRAG_OVER, diagramViewDragOver, false, EventPriority.DEFAULT, true);

    this.addEventListener(KeyboardEvent.KEY_DOWN, diagramEditorViewKeyDown, false, EventPriority.DEFAULT, true);

    this.addEventListener(MouseEvent.MOUSE_OUT, handleMouseOut, false, EventPriority.DEFAULT, true);

    this.addChild(elementContextPanel);
    elementContextPanel.id = "elementContextPanel";
    hideElementContextPanel();

    // create selection rectangle object
    selectionRect = new Canvas();
    selectionRect.setStyle("backgroundColor", "green");
    selectionRect.alpha = 0.5;

    diagramSelectionViewer = new DiagramEditorSelectionViewer(this);

    //create arrow
    arrow = new Arrow(new Point(0, 0), new Point(0, 0));

    registerStateHelpers();

    // go to default state
    this.currentState = "default";

  }

  protected function registerStateHelpers():void {
     // generate states
    var diagramStateUtil:DiagramEditorFlowStateUtil = createDiagramEditorFlowStateUtil();
    new DiagramEditorDefaultStateHelper(this).addStates();
    new DiagramEditorSelectingStateHelper(this).addStates();
    new DiagramEditorMovingStateHelper(this).addStates();
    new DiagramEditorCreatingFlowStateHelper(this, diagramStateUtil).addStates();
    new DiagramEditorCreatingElementStateHelper(this).addStates();
    new DiagramEditorMovingFlowStateHelper(this, diagramStateUtil).addStates();
  }

  protected function createDiagramEditorFlowStateUtil():DiagramEditorFlowStateUtil {
    return new DiagramEditorFlowStateUtil(this);
  }


  /**
   *
   * @param event
   */
  private function modelSelectedElementsChanged(event:Event):void {
    // event instance of CollectionEvent
    // check kind attribute for "add, delete, move..."
    // get added or removed child
    if (event is CollectionEvent) {
      var collectionEvent:CollectionEvent = event as CollectionEvent;
      // model elements changed.
      var changedElements:Array = collectionEvent.items;
      switch (collectionEvent.kind) {
        case CollectionEventKind.ADD:
          for each (var addedElement:Object in changedElements) {
            updateViewElement(addedElement, true);
          }
          // show context panel if a new element is selected.
          if (this._diagramEditorModel.selectedElements.length == 1) {
            showElementContextPanel();
          }
          break;

        case CollectionEventKind.REMOVE:
          for each (var removedElement:Object in changedElements) {
            updateViewElement(removedElement, false);
          }
          break;

        case CollectionEventKind.RESET:
          for each (var element:Object in _diagramEditorModel.diagramModel.elements) {
            updateViewElement(element, false);
          }
          break;
      }
    }
    // show context panel if possible
    showElementContextPanel();
  }

  /**
   * Handle keyBoardEvent
   * @param event
   */
  private function diagramEditorViewKeyDown(event:KeyboardEvent):void {
    var numberElement:int = _diagramEditorModel.selectedElements.length;

    if (event.keyCode == Keyboard.DELETE && numberElement > 0) {
      // remove temporary keydown listener
      this.removeEventListener(KeyboardEvent.KEY_DOWN, diagramEditorViewKeyDown);

      // create the windows to confirm deletion
      deleteConfirmWindow = ConfirmPopup(PopUpManager.createPopUp(Application.application as Application, ConfirmPopup, true));
      deleteConfirmWindow.title = "Warning : BPMN elements is to be deleted...";
      deleteConfirmWindow.addEventListener(CloseEvent.CLOSE, itemDeleteHandler);
      deleteConfirmWindow.addEventListener(ConfirmPopup.CANCEL, itemDeleteHandler);

      if (numberElement == 1) {
        deleteConfirmWindow.file.text = "Do you want to delete this element ?\rThis operation can't be undone.";
      }
      else {
        deleteConfirmWindow.file.text = "Do you want to delete this " + numberElement + " elements ?\rThis operation can't be undone.";
      }

      PopUpManager.centerPopUp(deleteConfirmWindow);
      deleteConfirmWindow.setFocus();
    }
  }

  private function itemDeleteHandler(event:CloseEvent):void {
    if (event.type == CloseEvent.CLOSE) {
      new DiagramEvent(
          DiagramEvent.SELECTED_DIAGRAM_ELEMENTS_DELETE,
          _diagramEditorModel).dispatch();
    }

    //add keydown listener temporary deleted during deleteConfirmWindow modal windows opening
    this.addEventListener(KeyboardEvent.KEY_DOWN, diagramEditorViewKeyDown);

  }

  /**
   * Hide the context panel for a diagram element
   */
  public function hideElementContextPanel():void {
    this.elementContextPanel.setVisible(false);
  }

  /**
   * Show the context panel for a diagram element:
   * set the panel attached view to the current selected object and show the panel.
   */
  public function showElementContextPanel():void {
    if (this.currentState == "default" && this._diagramEditorModel.selectedElements.length == 1 &&
        this._diagramEditorModel.selectedElements.getItemAt(0) is AbstractElement) {
      // get current element
      var selectedElement:AbstractElement = this._diagramEditorModel.selectedElements.getItemAt(0) as AbstractElement;
      if (selectedElement != null) {
        // get associated view
        var selectedElementView:AbstractElementView = _diagramView.getElementViewByModel(selectedElement);
        // set context panel attached view (updates the panel content and position)
        elementContextPanel.attachedView = selectedElementView;
        // reset context panel state
        elementContextPanel.currentState = "default";
        // show panel
        elementContextPanel.setVisible(true);
      }
    } else {
      hideElementContextPanel();
    }
  }

  /**
   * Called when an object is selected/unselected.
   */
  protected function updateViewElement(element:Object, isToBeSelected:Boolean):void {
    diagramSelectionViewer.updateViewElement(element, isToBeSelected);
  }

  /** Object drag handler
   *
   * Note: This method is designed to be abstract,
   * sub-classes MUST override it.
   * As Abstract method/classes support appears with Flex 4,
   * This method throw an CairngormError if called
   * directly.
   */
  protected function diagramViewDragEnter(event:DragEvent):void {
    //This method is designed to be abstract
    throw new CairngormError(CairngormMessageCodes.ABSTRACT_METHOD_CALLED);
  }

  /** Object drag handler
   *
   * Note: This method is designed to be abstract,
   * sub-classes MUST override it.
   * As Abstract method/classes support appears with Flex 4,
   * This method throw an CairngormError if called
   * directly.
   */
  protected function diagramViewDragOver(event:DragEvent):void {
    //This method is designed to be abstract
    throw new CairngormError(CairngormMessageCodes.ABSTRACT_METHOD_CALLED);
  }

  /**
   * Object drop handler.
   * This method is called when an object is droped in the diagram view

   *
   * Note: This method is designed to be abstract,
   * sub-classes MUST override it.
   * As Abstract method/classes support appears with Flex 4,
   * This method throw an CairngormError if called
   * directly.
   */
  protected function diagramViewDrop(event:DragEvent):void {
    //This method is designed to be abstract
    throw new CairngormError(CairngormMessageCodes.ABSTRACT_METHOD_CALLED);
  }

  /**
   * handler for mouse out event.
   * when the mouse leaves this view, the view goes back to the "default" state.
   */
  private function handleMouseOut(event:MouseEvent):void {
    // First check if we leave this view.
    // Mouse out events can be generated when the mouse goes over a child view of this view.

    // set target to the new object under the mouse.
    var target:InteractiveObject = event.relatedObject;
    // look recursively for a diagram element view
    // return when first element is found.
    while (target != null) {
      if (target is AbstractElementView || target == this) {
        // new object is a child of this view, return.
        return;
      }
      if (target == Application.application) {
        // we leave this view, go back to the "default" state.
        this.currentState = "default";
        return;
      } else if (target is DisplayObject) {
        // look in the parent
        target = (target as DisplayObject).parent;
      }
    }
  }

  public function get diagramView():AbstractDiagramView {
    return _diagramView;
  }

  public function get diagramEditorModel():DiagramEditorModel {
    return _diagramEditorModel;
  }

  public function get viewXOffset():int {
    return _viewXOffset;
  }

  public function get viewYOffset():int {
    return _viewYOffset;
  }
}
}
