/**
 * Bull SAS / OW2 Orchestra
 * Copyright (C) 2011 Bull S.A.S, and individual contributors as indicated
 * by the @authors tag.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */
package org.ow2.orchestra.designer.view.diagramEditor {
import flash.display.DisplayObject;
import flash.events.MouseEvent;
import flash.geom.Point;

import mx.events.FlexEvent;
import mx.states.AddChild;
import mx.states.SetEventHandler;
import mx.states.State;

import org.ow2.orchestra.designer.bpmn.model.AbstractElementWithPosition;
import org.ow2.orchestra.designer.bpmn.views.AbstractElementView;
import org.ow2.orchestra.designer.bpmn.views.utils.GraphicUtils;
import org.ow2.orchestra.designer.events.DiagramModificationEvent;

public class DiagramEditorCreatingElementStateHelper extends DiagramEditorStateHelper {


  public function DiagramEditorCreatingElementStateHelper(diagramEditorView:AbstractDiagramEditorView) {
    super(diagramEditorView);
  }

  /**
   * Generate "creating gateway" state.
   * In the creating flow state:
   * - the flow is drawn, from source element to mouse cursor
   * - on mouse move (while the button is down), the flow is updated : the head of the flow follows the mouse cursor.
   * - on clic, if the clic is on a compatible element, the flow is drawn between source and target elements . This view goes back to the "default" state.
   */
  public override function addStates():void {

    var state:State = new State();
    state.name = "creatingElement";
    diagramEditorView.states.push(state);

    var addArrowChild:AddChild = new AddChild(diagramEditorView, diagramEditorView.arrow);
    var addElementChild:AddChild = new AddChild(diagramEditorView, diagramEditorView.imageOfElementToAdd);
    state.overrides.push(addArrowChild);
    state.overrides.push(addElementChild);

    // handler for mouse move event
    var mouseMoveListener:SetEventHandler = new SetEventHandler(diagramEditorView, MouseEvent.MOUSE_MOVE);
    mouseMoveListener.handlerFunction = creatingElementMouseMove;
    state.overrides.push(mouseMoveListener);

    // handler for mouse click event
    var mouseClickListener:SetEventHandler = new SetEventHandler(diagramEditorView, MouseEvent.CLICK);
    mouseClickListener.handlerFunction = creatingElementMouseClick;
    state.overrides.push(mouseClickListener);

    // handler for state enter: resets the arrow position
    state.addEventListener(FlexEvent.ENTER_STATE, creatingElementStateEnter);
    state.addEventListener(FlexEvent.EXIT_STATE, creatingElementStateExit);
  }

  /**
   * handler for mouse move events when this view is in the "creating fProcesslow" state.
   */
  private function creatingElementStateEnter(event:FlexEvent):void {
    diagramEditorView.hideElementContextPanel();

    // Get mouse position.
    var mousePoint:Point = new Point(diagramEditorView.mouseX, diagramEditorView.mouseY);

    creatingElementMoveArrow(mousePoint);
  }

  private function creatingElementStateExit(event:FlexEvent):void {
  }

  /**
   * handler for mouse move events when this view is in the "creating flow" state.
   */
  private function creatingElementMouseMove(event:MouseEvent):void {
    // Get mouse event position. The coordinates are relative to the target element
    var targetLocalMousePoint:Point = new Point(event.localX, event.localY);
    // Transform the coordinates to be relative to this view
    var mousePoint:Point = diagramEditorView.globalToLocal(DisplayObject(event.target).localToGlobal(targetLocalMousePoint));

    creatingElementMoveArrow(mousePoint);
  }

  private function creatingElementMoveArrow(mousePoint:Point):void {
    // Get coordinates of the center of the source element
    var element:AbstractElementWithPosition = diagramEditorView.diagramEditorModel.selectedElements.getItemAt(0) as
        AbstractElementWithPosition;
    var sourceView:AbstractElementView = diagramEditorView.diagramView.getElementViewByModel(element);
    var sourceImage:DisplayObject = sourceView.borderImage;
    var sourcePoint:Point = GraphicUtils.findIntersection(
        sourceImage,
        diagramEditorView.diagramView.globalToLocal(sourceView.localToGlobal(new Point(sourceImage.x + sourceImage.width
            / 2, sourceImage.y + sourceImage.height / 2))),
        mousePoint,
        diagramEditorView.diagramView);

    diagramEditorView.imageOfElementToAdd.x = mousePoint.x - diagramEditorView.imageOfElementToAdd.width / 2;
    diagramEditorView.imageOfElementToAdd.y = mousePoint.y - diagramEditorView.imageOfElementToAdd.height / 2;
    var targetPoint:Point = GraphicUtils.findIntersection(
        diagramEditorView.imageOfElementToAdd,
        mousePoint,
        diagramEditorView.diagramView.globalToLocal(sourceView.localToGlobal(new Point(sourceImage.x, sourceImage.y))),
        diagramEditorView.diagramView);

    diagramEditorView.arrow.setSourceAndTarget(sourcePoint, targetPoint);
  }

  /**
   * handler for mouse click events when this view is in the "creating element" state.
   * Select the element under the mouse cursor and create a flow to this element.
   * The view goes back to the "default" state.
   */
  private function creatingElementMouseClick(event:MouseEvent):void {
    // Get mouse event position. The coordinates are relative to the target element
    var targetLocalMousePoint:Point = new Point(event.localX, event.localY);
    // Transform the coordinates to be relative to this view
    var mousePoint:Point = diagramEditorView.globalToLocal(DisplayObject(event.target).localToGlobal(targetLocalMousePoint));

    // check if mouse is on a element view or on the canvas.
    trace("creatingElementWithFlowMouseClick");

    // create a new element and a new sequenceFlow
    var elementSource:AbstractElementWithPosition =
        diagramEditorView.diagramEditorModel.selectedElements.getItemAt(0) as AbstractElementWithPosition;


    new DiagramModificationEvent(DiagramModificationEvent.DIAGRAM_ELEMENT_ADD_WITH_SEQUENCE,
        this.diagramEditorView.diagramEditorModel,
        diagramEditorView.modelOfElementToAdd,
        mousePoint.x - diagramEditorView.imageOfElementToAdd.width / 2,
        mousePoint.y - diagramEditorView.imageOfElementToAdd.height / 2,
        elementSource).dispatch();
    // go to default state
    diagramEditorView.currentState = "default";
  }
}
}